@@ -1181,10 +1181,7 @@ let rec trans_expr env evd e =
         in
         app_binop evd e binop a.(n - 2) prf1 a.(n - 1) prf2
       | d -> mkvar evd inj e
-  with Not_found ->
-    (* Feedback.msg_debug
-       Pp.(str "Not found " ++ Termops.Internal.debug_print_constr e); *)
-    mkvar evd inj e
+  with Not_found | DestKO ->    mkvar evd inj e
 
 let trans_expr env evd e =
   try pp_trans_expr env evd e (trans_expr env evd e)
